header {
/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * Project  Zero
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
 
package zero.core.config.antlr;
}


/* #################################################################### */
/* #                              Lexer                                 */
/* #################################################################### */
{
import antlr.TokenStreamSelector;
}
class StringLexer extends Lexer;

options {
	k=3;  // k=3 to distinguish GCKeyRef ${/ from VariableRef ${a
	charVocabulary = '\0'..'\ufffe';  // uffff is EOF
	defaultErrorHandler=false;
}

{
TokenStreamSelector _selector = null;

public void setSelector(TokenStreamSelector selector) {
	_selector = selector;
}
}

// This lexer is designed to work with the config lexer.
// _selector controls the multiplexed lexers.
Quote
	options { paraphrase = "'\"'"; }
	: '"' { _selector.pop(); }
	;

Reserved
	: ('$' '$') => "$$"! { $append("$"); $setType(Literal); }
	| (GCKeyRef { $setType(GCKeyRef); } 
	| VariableRef { $setType(VariableRef); } )
	;
	
Literal 
	: ( ~('\u0000'..'\u001f' | '\\' | '\"' | '$' ) | Escapes )+
	;

protected GCKeyRef
	: ("${" GCKey '}') {
		String ref = $getText;
		String name = ref.substring(2, ref.length()-1);
		$setText(name);
	}
	;
	
protected GCKey 
	: ('/' GCPathElement (('/') GCPathElement)*) ;
	
protected GCPathElement
	: (~('/' | '}' | '+' | '=' | ' '))+ ;
	
protected VariableRef
	: "${" Variable '}' {
		String ref = $getText;
		String name = ref.substring(2, ref.length()-1);
		$setText(name);
	}
	;
	
protected Variable
	: ('a'..'z' | 'A'..'Z') ('a'..'z' | 'A'..'Z' | '0'..'9' | '_')*
	;
	
protected Escapes
   	: ('\\' ('\"' | '\\' | '/' | 'b' | 'f' | 'n' | 'r' | 't' | ('u' HexDigit HexDigit HexDigit HexDigit))) {
   		String s = $getText;
   		$setText(s.substring(1));
   	}
   	;

protected HexDigit 
	: '0'..'9' | 'A'..'F' | 'a'..'f';
