header {
/*
 * ============================================================================
 * Licensed Materials - Property of IBM
 * Project  Zero
 *
 * (C) Copyright IBM Corp. 2007  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 * ============================================================================
 */
 
package zero.core.events.antlr;
}


/* #################################################################### */
/* #                              Lexer                                 */
/* #################################################################### */
{
import antlr.TokenStreamSelector;
}
class ConditionsLexer extends Lexer;

options {
	k=3;  // k=3 to distinguish GCKeyRef ${/ from VariableRef ${a
	charVocabulary = '\0'..'\ufffe';  // uffff is EOF
	defaultErrorHandler=false;
	importVocab=ValueLexer;
}

{
TokenStreamSelector _selector = null;

public void setSelector(TokenStreamSelector selector) {
	_selector = selector;
}
}


And
	options { paraphrase = "'&&'"; }
	: "&&"
	;
	
Or
	options { paraphrase = "'||'"; }
	: "||"
	;
	
Not
	options { paraphrase = "'!'"; }
	: "!"
	;

SelectorMatch
	options { paraphrase = "'=='"; }
	: "==" { _selector.push("value"); }
	;

SelectorNotMatch
	options { paraphrase = "'!='"; }
	: "!=" { _selector.push("value"); }
	;

RegexMatch
	options { paraphrase = "'=~'"; }
	: "=~" { _selector.push("value"); }
	;

RegexNotMatch
	options { paraphrase = "'!~'"; }
	: "!~" { _selector.push("value"); }
	;
	
LParen
	options { paraphrase = "'('"; }
	: '('
	;

RParen
	options { paraphrase = "')'"; }
	: ')'
	;

GCKey 
	: ('/' GCPathElement (('/') GCPathElement)*) ;
	
protected GCPathElement
	: (~('/' | '}' | '+' | '=' | ' ' | ')'))+ ;
	
WS 
	: ( ' ' ) { $setType(Token.SKIP); }
	;
	
	
/* #################################################################### */
/* #                             Parser                                 */
/* #################################################################### */
{
import antlr.TokenStreamRecognitionException;
import antlr.TokenStreamSelector;
}
class ConditionsParser extends Parser("zero.antlr.Parser");
	
options {
	k=6;
	buildAST=true; 	  // Automatically build the AST while parsing
}

tokens {
	Rule;
}

{
TokenStreamSelector _selector = null;

public void setSelector(TokenStreamSelector selector) {
	_selector = selector;
}
}


condition
	: GCKey ((SelectorMatch^ | SelectorNotMatch^ |RegexMatch^ | RegexNotMatch^) Value)?
	;
	
condition_predicate
	: condition_group ((And^ | Or^) condition_group)*
	;
	
condition_group
	: (Not^)? LParen! (condition | condition_predicate) RParen!
	;

rule
	: (condition | condition_predicate) EOF!
	;


/* #################################################################### */
/* #                             Walker                                 */
/* #################################################################### */
{
import java.util.logging.Level;
import java.util.logging.Logger;

import zero.core.events.*;
import zero.core.logging.LogHelper;
}
class ConditionsTreeWalker extends TreeParser;


{
private static final String CLASS_NAME = ConditionsTreeWalker.class.getName();
private static final Logger _logger = Logger.getLogger(CLASS_NAME, "zero.kernel.messages");


private String _fileName;

public void setFileName(String fileName) {
	_fileName = fileName;
}

public String getFileName() {
	return _fileName;
}


@Override
public void reportError(RecognitionException ex) {
	LogHelper.log(_logger, Level.SEVERE, CLASS_NAME, "reportError", ex.toString()); 
	throw new RuntimeException(ex);
}

}


condition returns [Condition condition = null]
{
String k, v;
Condition c1, c2;
}

	: #(Not c1=condition)
		{ condition = new NotCondition(c1); }
		
	| #(And c1=condition c2=condition)
		{ condition = new AndCondition(c1, c2); }
		
	| #(Or c1=condition c2=condition)
		{ condition = new OrCondition(c1, c2); }
	
	| #(SelectorMatch k=gcKey v=value)
		{ condition = new SelectorCondition(k, v); }
		
	| #(SelectorNotMatch k=gcKey v=value)
		{ condition = new NotCondition(new SelectorCondition(k, v)); }
		
	| #(RegexMatch k=gcKey v=value)
		{ condition = new RegexCondition(k, v); }
		
	| #(RegexNotMatch k=gcKey v=value)
		{ condition = new NotCondition(new RegexCondition(k, v)); }
		
	| (k=gcKey)
		{ condition = new ExistsCondition(k); }
	;
	
	
gcKey returns [String key = null]
	: t:GCKey { key = t.getText(); }
	;
	
value returns [String s = null]
	: v:Value { s = v.getText().trim(); }
	;
	